////////////////////////////
// Includes
////////////////////////////

#include <PA9.h>       // Include for PA_Lib
#include "gnugo.h"
#include "gfx/dsGo2.h"

#include "gfx/Sprites.pal.c"
#include "gfx/Sprites.raw.c"

// Sons
#include "stone.h"
#include "zoom.h"
#include "question.h"

////////////////////////////
// Defines
////////////////////////////

#define ZOOMSPEED	32 // Sempre han de ser multiples de 512
#define SCROLLSPEED	5	// Velocitat a la que es moura el scroll. Quan mes petita mes rapid
//#define SCREENX		256	// Tamany de la pantalla X
#define SCREENX		224	// Tamany de la pantalla X
#define SCREENY		192 // Tamany de la pantalla Y

#define MAPSIZE		1024	// Tamany de un eix del taulell en pixels. Se suposa que s quadrat

#define MAPFITXES	dsGo2_T0MapData3	// Nom del mapa a on es guardaran els grafics de les fitxes (del Tile Studio)
#define FITXESSIZE	128		// Tamany del mapa a on es guarden les fitxes (del Tile Studio)

#define SPRITE_PANEL	10	// Numero a on comencen els sprites del panel. Son 7 en total
#define SPRITE_STONE	21	// Numero del sprite de la pedra
#define SPRITE_STONE_Q	20	// Numero del sprite de la pedra amb interrogant
#define SPRITE_CONFIRM	0	// Numero a on comena els sprites de la confirmaci. 2 en total

#define SOUND_STONE		0	// So de deixar una pedra
#define SOUND_ZOOM		1	// So al fer zoom
#define SOUND_QUESTION	2	// So de una pregunta

#define S_NONE		0
#define S_WHITE		1
#define S_BLACK		2


////////////////////////////
// Typedef
////////////////////////////

typedef struct {
	short size;			// Tamany del taulell. Pot ser de 9,13 o 19 caselles.
	int scrollx;		// Posici X del scroll
	int scrolly;		// Posici Y del scroll
	int rotcenterx;		// Posici X del centre del zoom i de la rotaci
	int rotcentery;		// Posici Y del centre del zoom i de la rotaci
	short angle;		// Angle de la rotaci.
	int zoom;			// Zoom del taulell
	short cellx;		// Casella X respecte el punt 0
	short celly;		// Casella Y respecte el punt 0
	short pushedcellx;	// Casella X a on s'esta apretant amb el llapis
	short pushedcelly;	// Casella Y a on s'esta apretant amb el llapis
	short pixelstile;	// Numero de pixels que t una casella (en funci del zoom)
	short board[19][19];	// Caselles del taulell
	short turn;			// A on es guardar el torn actual
	bool waitingCPU;	// Ser true si el hum ha tirat i est esperant la resposta de la CPU
} StructBoard;


////////////////////////////
// Prototypes
////////////////////////////
void zoomIn(StructBoard *board);
void zoomOut(StructBoard *board);
void stone2tile(int x, int y, short stone, unsigned char boardStones[(MAPSIZE/8)][(MAPSIZE/8)], unsigned char mapFitxes[FITXESSIZE^2]);
void captureStylus(StructBoard *board);
void zoomBoard (int oldZoom, int newZoom, StructBoard *board, int cellx, int celly);
void drawPanel();
void initStone(StructBoard board);
void drawStone(StructBoard board);
void updateStone(StructBoard board);
void deleteStone();
void putStone(StructBoard *board, short stone);
// Dibuixa el taulell o actualitza la seva posici
void drawBoard(StructBoard board);
// Calcula els parametres del scroll
void calcScroll(StructBoard *board, short scrollIncH, short scrollIncV, int *incX, int *incY, int *newScrollX, int *newScrollY, short *newCellX, short *newCellY, short scrollSpeed);
// Fa un scroll del taulell
void doScroll(StructBoard *board, short scrollIncH, short scrollIncV);
// Mostra el text de debug
void printDebugText(int zoom, int scrollx, int scrolly, int rotcenterx, int rotcentery, int angle, int cellx, int celly, int pixelstile, short pushedcellx, short pushedcelly);
void printBoardText(StructBoard board);
short calcPixelsTile(int zoom);
void Board_Init(StructBoard *board);
bool inside(int touch[]);
void playSound(short sound);
bool isValidMove(StructBoard board);
bool confirmPutStone(StructBoard board);
void initSprites();
void syncBoard(StructBoard board);
void syncBoardGnugo(StructBoard *board);
void gnugo_Init();

////////////////////////////
// Global Variables
////////////////////////////

unsigned char boardStones[(MAPSIZE/8)][(MAPSIZE/8)] __attribute__ ((aligned (4)));

short lastSoundChan = 0;

// Definicions de les arees a on es pot apretar amb la stylus
// [X,Y,Ample,Alt]
int touchPanel[4] = {224,0,32,192};
int touchPanelZoomIn[4] = {224,32,32,32};
int touchPanelZoomOut[4] = {224,96,32,32};
int touchConfirmYes[4] = {0-10,16,32,16+10};
int touchConfirmNo[4] = {32-10,16,32,16+10};
//int touchConfirmMove[4] = {0,16,64,16};

////////////////////////////
// Main
////////////////////////////

int main(int argc, char ** argv) {
	////////////////////////////
	// Variables
	////////////////////////////

	StructBoard board; // Variable a on guardem la informaci del taulell
	
	// Variables que controlen el scroll
	short scrollIncH; // Increment Horitzontal
	short scrollIncV; // Increment Vertical
	

		
	////////////////////////////
	// Main
	////////////////////////////

	PA_Init();    // Initializes PA_Lib
	PA_InitVBL(); // Initializes a standard VBL
	dsGo2_Init(); // Inicialitzem els grafics
	
	// Load Palettes
	PA_LoadPal(PAL_BG0, dsGo2_Palette1_Pal); // Backgrounds
	PA_LoadPal(PAL_BG1, dsGo2_Palette1_Pal); // Backgrounds
	PA_LoadPal(PAL_SPRITE0, Sprites_Palette); // Sprites
	
	// Posem el mode de video per fer rotacions i zooms a la pantalla d'abaix
	PA_SetVideoMode(0, 2);
	
	// Carreguem el fondo
	PA_LoadRotBg(0, 3, dsGo2_Tiles1_Tiles, dsGo2_T0MapData0, BG_ROT_1024X1024, 1);
	
	Board_Init(&board);

	// Carreguem el fondo amb les fitxes
	PA_LoadRotBg(0, 2, dsGo2_Tiles1_Tiles, boardStones, BG_ROT_1024X1024, 0);

	PA_InitText(1, 0);
	drawBoard(board);
	initSprites();
	
	syncBoard(board);
	gnugo_Init();
	
	// Infinite loop to keep the program running
	while (1)
	{
		// Inicialitzem les variables de l'increment de zoom
		scrollIncH = 0; 
		scrollIncV = 0;
		
		printBoardText(board);
		
		// Captura la pantalla tactil i ho posa a les variables pushedcell de board
		captureStylus(&board);
	
		// Captura les tecles
		if (board.zoom != 1024) {
			scrollIncH += (Pad.Held.Right - Pad.Held.Left);
			scrollIncV += (Pad.Held.Down - Pad.Held.Up);
		}
		if (Pad.Released.X) {
			zoomIn(&board);
		} else if (Pad.Released.Y) {
			zoomOut(&board);
		}
		
		// Fa el scroll
		if ((scrollIncH) || (scrollIncV)) {
			doScroll(&board, scrollIncH, scrollIncV);
		}


		
		//printDebugText(board.zoom, board.scrollx, board.scrolly, board.rotcenterx, board.rotcentery, board.angle, board.cellx, board.celly, board.pixelstile,board.pushedcellx, board.pushedcelly);
		
		syncBoard(board);
		
		
		drawBoard(board);
		
		PA_CheckLid();
		PA_WaitForVBL();
		
		if (board.waitingCPU) { // Si acabem de tirar farem que tiri gnugo
			board.waitingCPU = false;
			examboard(mymove); // Retira les peces mortes
			genmove(&i,&j);
			p[i][j] = mymove;
			board.pushedcellx = i;
			board.pushedcelly = j;
			examboard(umove);
			syncBoardGnugo(&board);
			putStone(&board,mymove);	
			drawBoard(board);
			PA_WaitForVBL();
		}
		
	}
	
	return 0;
} // End of main()


////////////////////////////
// Functions
////////////////////////////
void zoomIn(StructBoard *board) {
	switch (board->zoom) {
		case 1024:
			zoomBoard(1024,512,board, board->pushedcellx, board->pushedcelly);
			break;
		case 512:
			zoomBoard(512,256,board, board->pushedcellx, board->pushedcelly);
			break;
		case 256:
			break;
	}
}

void zoomOut(StructBoard *board) {
	switch (board->zoom) {
		case 1024:
			break;
		case 512:
			// Fem zoom al centre de la pantalla
			zoomBoard(512,1024,board, SCREENX/2/board->pixelstile + board->cellx,SCREENY/2/board->pixelstile + board->celly); 
			break;
		case 256:
			zoomBoard(256,512,board, board->pushedcellx, board->pushedcelly);
			break;
	}
}

void stone2tile(int x, int y, short stone, unsigned char boardStones[(MAPSIZE/8)][(MAPSIZE/8)], unsigned char mapFitxes[FITXESSIZE^2]) {
	int n, m;
	
	for (n=0; n < 4; n++) {
		for (m=0; m < 4; m++) {
			boardStones[x*4+n][y*4+m] = mapFitxes[m + n*FITXESSIZE + stone*4];
		}
	}
}

void captureStylus(StructBoard *board) {
	
	// Tractarem diferent quan apretem sobre la pantalla o sobre el panel
	if (inside(touchPanel)) {
		if (Stylus.Released) {
			// Si es toca el panel de joc
			// Esborrem la pedra per si de cas ens hem desplaat desde el taulell
			deleteStone();
			if (inside(touchPanelZoomIn)) {
				zoomIn(board);
			} else if (inside(touchPanelZoomOut)) {
				zoomOut(board);
			}
		}
	} else {
		if ((Stylus.Held) || (Stylus.Newpress)) {
			// Si no es toca el panel, posicionem la fitxa
			board->pushedcellx = Stylus.X/board->pixelstile + board->cellx;
			board->pushedcelly = Stylus.Y/board->pixelstile + board->celly;
		}
		switch (board->zoom) {
			case 1024:
				if (Stylus.Released) {
					zoomIn(board);
				}
				break;
			case 512:
			case 256:
				// Si el zoom < 1024 posa la fitxa al taulell si esta dins dels limits
				if ((board->pushedcellx >=0) && (board->pushedcellx < board->size) && 
					(board->pushedcelly >=0) && (board->pushedcelly < board->size)) {
					if (Stylus.Newpress) {
						drawStone(*board);
					}
					if (Stylus.Held) {
						updateStone(*board);
					}
					if (Stylus.Released) {
						// Mirem que sigui un moviment valid
						if (isValidMove(*board)) {
							// Demanarem confirmaci
							if (confirmPutStone(*board)) {
								// Marcarem el flag conforme estem esperant resposta de la CPU
								board->waitingCPU = true;
								putStone(board,umove);
							}
						}
						deleteStone();
					}
				}
				break;
		}
	}
}

void zoomBoard (int oldZoom, int newZoom, StructBoard *board, int cellx, int celly) {
	int x=0;
	short cellcenterx, cellcentery; // la casella a on es far el zoom.
	short centertilex, centertiley; // la casella que estar al centre de la pantalla
	
	playSound(SOUND_ZOOM);
	if (oldZoom > newZoom) {
	// Si s'acosta
		// Calculem el tile que estar centre de la pantalla 
		centertilex = SCREENX / calcPixelsTile(newZoom) / 2;
		centertiley = SCREENY / calcPixelsTile(newZoom) / 2;
		
		// Calculem la casella que ser el centre de la zona ampliada i 
		// evitem que surti fora dels limits del taulell
		if (cellx < centertilex) {
			cellcenterx = 0;
		} else if (cellx > board->size - centertilex) {
			cellcenterx = board->size - centertilex*2;
		} else {
			cellcenterx = cellx - centertilex;
		}
		
		if (celly < centertilex) {
			cellcentery = 0;
		} else if (celly > board->size - centertiley) {
			cellcentery = board->size - centertiley*2;
		} else {
			cellcentery = celly - centertiley;
		}
		
		// Calculem el centre de rotaci a partir del centre en tiles
		board->rotcenterx = (cellcenterx - board->cellx) * calcPixelsTile(newZoom);
		board->rotcentery = (cellcentery - board->celly) * calcPixelsTile(newZoom);
	} else {
	// Si s'allunya
		if (newZoom == 1024) {
		// Si el nou zoom = 1024, centrarem el taulell
			board->rotcenterx = calcPixelsTile(oldZoom) * ( (((SCREENX - (board->size * calcPixelsTile(newZoom))) / calcPixelsTile(newZoom)) / 2) + board->cellx);
			board->rotcentery = calcPixelsTile(oldZoom) * ( (((SCREENY - (board->size * calcPixelsTile(newZoom))) / calcPixelsTile(newZoom)) / 2) + board->celly);
		} 
	}
	
	// Calculem el nou valor del scroll basat en el nou rotcenter. Posem a x les dades que no ens interessen
	calcScroll(board, 0, 0, &x, &x, &board->scrollx, &board->scrolly, &board->cellx, &board->celly, SCROLLSPEED);
	
	if (oldZoom > newZoom) {
		// Apropa
		for (x=oldZoom ; x >= newZoom; x=x-ZOOMSPEED) {
			board->zoom = x;
			drawBoard(*board);
			PA_WaitForVBL();
		}
		// Actualitzem les variables que es modifiquen
		board->cellx = (board->rotcenterx / calcPixelsTile(newZoom)) + board->cellx;
		board->celly = (board->rotcentery / calcPixelsTile(newZoom)) + board->celly;
	} else {
		// Allunya
		for (x=oldZoom ; x <= newZoom; x=x+ZOOMSPEED) {
			board->zoom = x;
			drawBoard(*board);
			PA_WaitForVBL();
		}
		// Actualitzem les variables que es modifiquen
		board->cellx = ( (- board->rotcenterx) / calcPixelsTile(oldZoom)) + board->cellx;
		board->celly = ( (- board->rotcentery) / calcPixelsTile(oldZoom)) + board->celly;
	}

	// Actualitzem les variables que es modifiquen
	board->pixelstile =  calcPixelsTile(board->zoom);
}



void drawStone(StructBoard board) {
	PA_UpdateSpriteGfx(0, SPRITE_STONE, (void*)&Sprites_Bitmap[1024 * board.turn]);
	updateStone(board);
}


void updateStone(StructBoard board) {
	PA_SetSpriteXY(0,SPRITE_STONE, 
		(board.pushedcellx - board.cellx) * board.pixelstile - ((board.zoom-256)/(board.zoom/16)), 
		(board.pushedcelly - board.celly) * board.pixelstile - ((board.zoom-256)/(board.zoom/16)));
	if (board.zoom == 512) {
		PA_SetRotset(0,1,512-board.angle,board.zoom-128,board.zoom-128);
	} else {
		PA_SetRotset(0,1,512-board.angle,board.zoom,board.zoom);
	}
}

void deleteStone() {
	//PA_DeleteSprite(0,SPRITE_STONE);
	PA_SetSpriteXY(0,SPRITE_STONE, SCREENX,SCREENY);
}

// Guarda la pedra al taulell
void putStone(StructBoard *board, short stone) {
	// Primer actualitzem la taula a on guardarem el taulell
	board->board[board->pushedcellx][board->pushedcelly] = stone;

	// Actualitzem el tilemap
	stone2tile(board->pushedcelly, board->pushedcellx, board->board[board->pushedcellx][board->pushedcelly],boardStones, (unsigned char *)MAPFITXES);
	// Actualitzem el background
	//PA_LoadRotBg(0, 2, dsGo2_Tiles1_Tiles, boardStones, BG_ROT_1024X1024, 0);
	PA_LoadRotBgMap(0, 2, boardStones, BG_ROT_1024X1024);
	drawBoard(*board);
	playSound(SOUND_STONE);
}

void drawBoard(StructBoard board) {
	PA_SetBgRot(0, 3, board.scrollx, board.scrolly, board.rotcenterx, board.rotcentery, board.angle, board.zoom);
	PA_SetBgRot(0, 2, board.scrollx, board.scrolly, board.rotcenterx, board.rotcentery, board.angle, board.zoom);
}

// Donat un increment horitzontal i vertical, modifica les variables:
//	 incX/Y (lo que s'ha de sumar al scroll actual depenent de la velocitat del scroll)
//	newScrollX/Y nou scroll un cop fet el moviment
//	newCellX/Y	nova casella despres d'haver fet el scroll
void calcScroll(StructBoard *board, short scrollIncH, short scrollIncV, int *incX, int *incY, int *newScrollX, int *newScrollY, short *newCellX, short *newCellY, short scrollSpeed) {
	int limitX, limitY;
	// Calculem fins a on s'ha d'arribar
	*newCellX = board->cellx + scrollIncH;
	*newCellY = board->celly + scrollIncV;
	
	// No permetrem que el taulell surti fora de la pantalla
	
	// Calculem els limits del taulell
	limitX = board->size - (SCREENX / board->pixelstile);
	limitY = board->size - (SCREENY / board->pixelstile);
	
	
	// Evita que el taulell surti fora de la pantalla
	
	// Es compleix quan el taulell surt per l'esquerra i no hi cap tot a la pantalla
	if ((*newCellX > 0) && (*newCellX > limitX) && (board->pixelstile * board->size > SCREENX) ) {
		*newCellX = limitX;
	// Es compleix quan el taulell surt per la dreta i no hi cap tot a la pantalla
	} else if ((*newCellX < 0) && (*newCellX < limitX) && (board->pixelstile * board->size > SCREENX)) {
		*newCellX = 0;
	// Es compleix quan el taulell surt per l'esquerra i hi cap tot a la pantalla
	} else if ((*newCellX > 0) && (*newCellX > limitX) && (board->pixelstile * board->size < SCREENX) ) {
		*newCellX = 0;
	// Es compleix quan el taulell surt per la dreta i hi cap tot a la pantalla
	} else if ((*newCellX < 0) && (*newCellX < limitX) && (board->pixelstile * board->size < SCREENX)) {
		*newCellX = limitX;
	}
	
	// Es compleix quan el taulell surt per dalt i no hi cap tot a la pantalla
	if ((*newCellY > 0) && (*newCellY > limitY) && (board->pixelstile * board->size > SCREENY) ) {
		*newCellY = limitY;
	// Es compleix quan el taulell surt per la dalt i no hi cap tot a la pantalla
	} else if ((*newCellY < 0) && (*newCellY < limitY) && (board->pixelstile * board->size > SCREENY)) {
		*newCellY = 0;
	// Es compleix quan el taulell surt per baix i hi cap tot a la pantalla
	} else if ((*newCellY > 0) && (*newCellY > limitY) && (board->pixelstile * board->size < SCREENY) ) {
		*newCellY = 0;
	// Es compleix quan el taulell surt per baix i hi cap tot a la pantalla
	} else if ((*newCellY < 0) && (*newCellY < limitY) && (board->pixelstile * board->size < SCREENY)) {
		*newCellY = limitY;
	}

	*newScrollX = ( board->rotcenterx  + *newCellX * board->pixelstile - SCREENX*(*newCellX/board->size) ) * board->zoom/256;
	*newScrollY = ( board->rotcentery  + *newCellY * board->pixelstile - SCREENY*(*newCellY/board->size) ) * board->zoom/256;
	// Calculem l'increment
	*incX = (*newScrollX - board->scrollx) / scrollSpeed;
	*incY = (*newScrollY - board->scrolly) / scrollSpeed;
}

void doScroll(StructBoard *board, short scrollIncH, short scrollIncV) {
	int incX, incY;
	int newScrollX, newScrollY;
	short newCellX, newCellY;
	int x;
	
	deleteStone();
	calcScroll(board, scrollIncH, scrollIncV, &incX, &incY, &newScrollX, &newScrollY, &newCellX, &newCellY, SCROLLSPEED);
	
	// fem el loop per mostrar el zoom
	for ( x=SCROLLSPEED ; x>0 ; x--) {
		board->scrollx = board->scrollx + incX;
		board->scrolly = board->scrolly + incY;
		drawBoard(*board);
		//printDebugText(board->zoom, board->scrollx, board->scrolly, board->rotcenterx, board->rotcentery, board->angle, newCellX, newCellY, board->pixelstile, 0,0);
		PA_WaitForVBL();
	}
	
	// Actualitzem les variables que es modifiquen
	board->cellx = newCellX;
	board->celly = newCellY;
	board->scrollx = newScrollX;
	board->scrolly = newScrollY;
}

void printDebugText(int zoom, int scrollx, int scrolly, int rotcenterx, int rotcentery, int angle, int cellx, int celly, int pixelstile, short pushedcellx, short pushedcelly) {
	PA_OutputSimpleText(1, 2, 2, "Zoom       : Start/Select");
	PA_OutputSimpleText(1, 2, 3, "ScrollX    : Left/Right");
	PA_OutputSimpleText(1, 2, 4, "Scrolly    : Up/Down");
	PA_OutputSimpleText(1, 2, 5, "RotCenterX : A/Y");
	PA_OutputSimpleText(1, 2, 6, "RotCenterY : B/X");
	PA_OutputSimpleText(1, 2, 7, "Angle      : R/L");

	PA_OutputText(1, 2, 8, "Zoom       :   %d     ",zoom);
	PA_OutputText(1, 2, 9, "ScrollX    :   %d     ",scrollx);
	PA_OutputText(1, 2, 10, "Scrolly    :   %d     ",scrolly);
	PA_OutputText(1, 2, 11, "RotCenterX :   %d     ",rotcenterx);
	PA_OutputText(1, 2, 12, "RotCenterY :   %d     ",rotcentery);
	PA_OutputText(1, 2, 13, "Angle      :   %d     ",angle);
	PA_OutputText(1, 1, 14, "Stylus Position : %d, %d   ", Stylus.X, Stylus.Y);
	PA_OutputText(1, 1, 15, "Cell : %d,%d      ", cellx, celly);
	PA_OutputText(1, 1, 16, "Pixelstile : %d   ", pixelstile);
	PA_OutputText(1, 1, 17, "Stylus : %d-%d-%d   ", Stylus.Newpress, Stylus.Held, Stylus.Released);
	PA_OutputText(1, 1, 18, "Pushedcell(XY) : %d - %d   ", pushedcellx, pushedcelly);
	
}

void printBoardText(StructBoard board) {
	int x, y;
	
	PA_OutputSimpleText(1, 1, 0, "  1234567890123456789");
	for (y=0; y<19; y++) {
		PA_OutputText(1,0,y+2, "%d", y);
	}
	
	for (x=0; x<19; x++) {
		for (y=0; y<19; y++) {
			//PA_OutputText(1,x+3,y+2, "%d", board.board[x][y]);
			PA_OutputText(1,x+3,y+2, "%d", p[x][y]);
		}
	}
	
	PA_OutputText(1, 1, 21, "Pushedcell(XY) : %d - %d   ", board.pushedcellx, board.pushedcelly);
	PA_OutputText(1, 1, 22, "Zoom       :   %d     ",board.zoom);
	PA_OutputText(1, 1, 23, "waitingCPU       :   %d     ",board.waitingCPU);
}

short calcPixelsTile(int zoom) {
	return 32 / ( zoom / 256 );
}

void Board_Init(StructBoard *board) {
	int x, y;
	
	// Inicialitzem variables
	board->waitingCPU = false;
	board->size = 19;
	board->angle = 0;
	board->zoom = 1024;
	board->rotcenterx = SCREENX/2; // Tamany en pixels de la pantalla eix x / 2
	board->rotcentery = SCREENY/2; // Tamany en pixels de la pantalla eix y / 2
	board->pushedcellx = 0;
	board->pushedcelly = 0;
	board->pixelstile =  calcPixelsTile(board->zoom);
	board->cellx = - ((SCREENX - (board->size * board->pixelstile)) / board->pixelstile) / 2; //Centrem
	board->celly = - ((SCREENY - (board->size * board->pixelstile)) / board->pixelstile) / 2; //Centrem el taulell
	board->turn = S_BLACK;
	// Inicialitzem el taulell 
	for (x=0; x < 19; x++) {
		for (y=0; y < 19; y++) {
			board->board[x][y] = S_NONE;
		}
	}
	// Actualitzem els valors del scroll
	calcScroll(board, 0, 0, &x, &x, &board->scrollx, &board->scrolly, &board->cellx, &board->celly, SCROLLSPEED);
	
	// Inicialitzem el fondo a on guardarem les fitxes
	for (x=0; x < 32; x++) {
		for (y=0; y < 32; y++) {
			stone2tile(x,y, S_NONE,boardStones, (unsigned char *)MAPFITXES);
		}
	}
}
bool inside(int toucharea[]) {
	if ((Stylus.X > toucharea[0]) && 
		(Stylus.X < toucharea[0]+toucharea[2]) && 
		(Stylus.Y > toucharea[1]) &&
		(Stylus.Y < toucharea[1]+toucharea[3])) {
			return true;
		} else {
			return false;
		}
}

void playSound(short sound) {
	switch (sound) {
		case SOUND_STONE:
			PA_PlaySound(lastSoundChan, (void*)stone, (u32)stone_size,120,22050);
			break;
		case SOUND_ZOOM:
			PA_PlaySound(lastSoundChan, (void*)zoom, (u32)zoom_size,120,22050);
			break;
		case SOUND_QUESTION:
			PA_PlaySound(lastSoundChan, (void*)question, (u32)question_size,120,22050);
			break;
	}
	
	if (lastSoundChan < 3)
		lastSoundChan++;
	else
		lastSoundChan = 0;
}


bool isValidMove(StructBoard board) {
	bool valid;
	
	// Verifiquem que no s'estigui posant sobre cap altre fitxa
	if (board.board[board.pushedcellx][board.pushedcelly] > 0) {
		valid = false;
	} else {
		valid = true;
	}
	
	// TODO: Fer que no es pugui posar quan est en atari
	
	return valid;
}

bool confirmPutStone(StructBoard board) {
	int x, y;
	bool exit = false;
	int newTouchConfirmYes[4];
	int newTouchConfirmNo[4];
	bool response = false;
	
	// Primer calculem a on ho posicionarem
	x = (board.pushedcellx - board.cellx) * board.pixelstile;
	y = (board.pushedcelly - board.celly) * board.pixelstile -32 -5;

	// Coloquem la pedra amb l'interrogant
	PA_UpdateSpriteGfx(0, SPRITE_STONE_Q, (void*)&Sprites_Bitmap[1024 * (board.turn +8)]);
	PA_SetSpriteXY(0,SPRITE_STONE_Q, 
		(board.pushedcellx - board.cellx) * board.pixelstile - ((board.zoom-256)/(board.zoom/16)), 
		(board.pushedcelly - board.celly) * board.pixelstile - ((board.zoom-256)/(board.zoom/16)));
		if (board.zoom == 512) {
			PA_SetRotset(0,2,512-board.angle,board.zoom-128,board.zoom-128);
		} else {
			PA_SetRotset(0,2,512-board.angle,board.zoom,board.zoom);
		}

	while (!exit) {
		// Evitem que surti fora de la pantalla i que no es posi a sobre del panel
		if (x <0) {
			x = 0;
		} else if (x+64 > SCREENX) {
			x = SCREENX-64;
		}
		if (y < 0) {
			y = (board.pushedcelly - board.celly + 1)*board.pixelstile;
		} else if (y+32 > SCREENY) {
			y = SCREENY - 32 - board.pixelstile;
		}
	
		PA_SetSpriteXY(0,SPRITE_CONFIRM, x, y);
		PA_SetSpriteXY(0,SPRITE_CONFIRM+1, x+32, y);

		// Calculem les noves zones a on posarem el stylus
		newTouchConfirmYes[0] = touchConfirmYes[0] + x;
		newTouchConfirmYes[1] = touchConfirmYes[1] + y;
		newTouchConfirmYes[2] = touchConfirmYes[2];
		newTouchConfirmYes[3] = touchConfirmYes[3];
		newTouchConfirmNo[0] = touchConfirmNo[0] + x;
		newTouchConfirmNo[1] = touchConfirmNo[1] + y;
		newTouchConfirmNo[2] = touchConfirmNo[2];
		newTouchConfirmNo[3] = touchConfirmNo[3];

		
		if (Stylus.Newpress) {
			if (inside(newTouchConfirmYes)) {
				PA_UpdateSpriteGfx(0, SPRITE_CONFIRM, (void*)&Sprites_Bitmap[1024 * 5]);
			} else if (inside(newTouchConfirmNo)) {
				PA_UpdateSpriteGfx(0, SPRITE_CONFIRM+1, (void*)&Sprites_Bitmap[1024 * 6]);
			}
		} else if (Stylus.Released) {
			if (inside(newTouchConfirmYes)) {
				PA_UpdateSpriteGfx(0, SPRITE_CONFIRM, (void*)&Sprites_Bitmap[1024 * 3]);
				PA_SetSpriteXY(0,SPRITE_CONFIRM, SCREENX, SCREENY);
				PA_SetSpriteXY(0,SPRITE_CONFIRM+1, SCREENX, SCREENY);
				response = true;
				exit = true;
			} else if (inside(newTouchConfirmNo)) {
				PA_UpdateSpriteGfx(0, SPRITE_CONFIRM+1, (void*)&Sprites_Bitmap[1024 * 4]);
				PA_SetSpriteXY(0,SPRITE_CONFIRM, SCREENX, SCREENY);
				PA_SetSpriteXY(0,SPRITE_CONFIRM+1, SCREENX, SCREENY);
				response = false;
				exit = true;
			}
		} else if (Stylus.Held) {
			if (!inside(newTouchConfirmYes)) {
				PA_UpdateSpriteGfx(0, SPRITE_CONFIRM, (void*)&Sprites_Bitmap[1024 * 3]);
			}
			if (!inside(newTouchConfirmNo)) {
				PA_UpdateSpriteGfx(0, SPRITE_CONFIRM+1, (void*)&Sprites_Bitmap[1024 * 4]);
			}
		}
		
		PA_CheckLid();
		PA_WaitForVBL();
	}
	// Esborrem la pedra amb l'interrogant
	PA_SetSpriteXY(0,SPRITE_STONE_Q, SCREENX,SCREENY);
	return response;
}


void initSprites() {
	// Inicialitzem els sprites. Inicialment els posem fora de la pantalla perque no es vegin
	short x;
	
	// Creem el sprite de la pedra que deixem
	PA_CreateSprite(0, SPRITE_STONE, (void*)&Sprites_Bitmap[0] , OBJ_SIZE_32X32,1, 0, SCREENX, SCREENY);
	PA_SetSpriteRotEnable(0, SPRITE_STONE, 1);
	
	PA_CreateSprite(0, SPRITE_STONE_Q, (void*)&Sprites_Bitmap[1024*9] , OBJ_SIZE_32X32,1, 0, SCREENX, SCREENY);
	PA_SetSpriteRotEnable(0, SPRITE_STONE_Q, 2);

	
	// Creem el panel
	for (x=0; x<6; x++) {
		PA_CreateSprite(0, SPRITE_PANEL+x, (void*)&Sprites_Bitmap[1024 * (7*x + 7+x)] , OBJ_SIZE_32X32,1, 0, 32*7, 32*x);
		PA_SetSpriteRotEnable(0, SPRITE_PANEL+x, 0);
	}
	
	// Creem el panel de confirmaci
	PA_CreateSprite(0, SPRITE_CONFIRM, (void*)&Sprites_Bitmap[1024 * 3] , OBJ_SIZE_32X32,1, 0, SCREENX, SCREENY);
	PA_CreateSprite(0, SPRITE_CONFIRM+1, (void*)&Sprites_Bitmap[1024 * 4] , OBJ_SIZE_32X32,1, 0, SCREENX, SCREENY);

}


// Funcions de gnugo

void syncBoard(StructBoard board) {
	int x,y;
	
	for (x=0; x<19; x++) {
		for (y=0; y<19; y++) {
			p[x][y] = board.board[x][y];
		}
	}
}

void syncBoardGnugo(StructBoard *board) {
	int x,y;
	
	for (x=0; x<19; x++) {
		for (y=0; y<19; y++) {
			board->board[x][y] = p[x][y];
		}
	}

}
void gnugo_Init(){
	
	for (i = 0; i < 9; i++)
		opn[i] = 1;
	opn[4] = 0;

	/* init board */
	for (i = 0; i < 19; i++)
		for (j = 0; j < 19; j++)
			p[i][j] = EMPTY;
	/* init global variables */
    mk = 0;  uk = 0;
	play = 1;
	pass = 0;
	mik = -1; mjk = -1;
	uik = -1; ujk = -1;
   
	handicap = 0;
	sethand(handicap);
	mymove = S_WHITE;   /* computer white */
	umove = S_BLACK;   /* human black */
}
